/*********************************************************************
 *                                                                   *
 *                     XGS : Apple IIGS Emulator                     *
 *                                                                   *
 *        Written and Copyright (C)1996 by Joshua M. Thompson        *
 *                                                                   *
 *  You are free to distribute this code for non-commercial purposes *
 * I ask only that you notify me of any changes you make to the code *
 *     Commercial use is prohibited without my written permission    *
 *                                                                   *
 *********************************************************************/

/*
 * File: emul.c
 *
 * This is the startup/shutdown code for the emulator. It's main
 * purpose is to initialize the other emulator subsystems and then
 * pass control to the 65816 to begin emulation. It also contains
 * the EMUL_Update() routine called periodically by the 65816; this
 * routine in turn calls the update routines of the other subsystems.
 */

#include <xgs.h>
#include <stdlib.h>
#include "emul.h"
#include "../adb/adb.h"
#include "../clock/clock.h"
#include "../cpu/m65816.h"
#include "../disks/disks.h"
#include "../memory/memory.h"
#include "../sound/sound.h"
#include "../video/video.h"
#include <time.h>

int EMUL_init()
{
	int	err;

	printf("%s\n",VERSION);
	printf("Written and (C) 1996 by Joshua M. Thompson\n");
	if ((err = MEM_init())) return err;
	if ((err = VID_init())) {
		MEM_shutdown();
		return err;
	}
	if ((err = SND_init())) {
		VID_shutdown();
		MEM_shutdown();
		return err;
	}
	if ((err = ADB_init())) {
		SND_shutdown();
		VID_shutdown();
		MEM_shutdown();
		return err;
	}
	if ((err = CLK_init())) {
		ADB_shutdown();
		SND_shutdown();
		VID_shutdown();
		MEM_shutdown();
		return err;
	}
	if ((err = DSK_init())) {
		CLK_shutdown();
		ADB_shutdown();
		SND_shutdown();
		VID_shutdown();
		MEM_shutdown();
		return err;
	}
	return 0;
}

void EMUL_update()
{
/*
	if (PC.A == 0x002000) EMUL_trace(1);
	if (PC.A == 0x002259) EMUL_shutdown();
*/
	MEM_update();
	VID_update();
	SND_update();
	ADB_update();
	CLK_update();
	DSK_update();
}

void EMUL_reset()
{
	MEM_reset();
	VID_reset();
	SND_reset();
	ADB_reset();
	CLK_reset();
	DSK_reset();
	M65816_reset();
}

void EMUL_shutdown()
{
	DSK_shutdown();
	CLK_shutdown();
	ADB_shutdown();
	SND_shutdown();
	VID_shutdown();
	MEM_shutdown();
	exit(0);
}

void EMUL_trace(int parm)
{
	M65816_trace(parm);
}

void EMUL_handleWDM(byte parm)
{
	switch(parm) {
		case 0xC5 :	DSK_handleSlot5();
				break;
		case 0xC7 :	DSK_handleSlot7();
				break;
		case 0xFD :	M65816_trace(0);
				break;
		case 0xFE :	M65816_trace(1);
				break;
		case 0xFF :	EMUL_shutdown();
				break;
		default :	break;
	}
}

void usage(char *myname)
{
	fprintf(stderr,"\nUsage: %s [-s5d1 filename ] [-s5d2 filename ] [-s6d1 filename ] [-s6d2 filename ] [-s7d1 filename ] [-s7d2 filename ] [-trace]\n",myname);
	exit(1);
}

void main(int argc, char *argv[])
{
	int	err,i;

	i = 0;
	while (++i < argc) {
		if (!strcmp(argv[i],"-trace")) {
			EMUL_trace(1);
		} else if (!strcmp(argv[i],"-s5d1")) {
			if (++i >= argc) usage(argv[0]);
			s5d1_startup = argv[i];
		} else if (!strcmp(argv[i],"-s5d2")) {
			if (++i >= argc) usage(argv[0]);
			s5d2_startup = argv[i];
		} else if (!strcmp(argv[i],"-s6d1")) {
			if (++i >= argc) usage(argv[0]);
			s6d1_startup = argv[i];
		} else if (!strcmp(argv[i],"-s6d2")) {
			if (++i >= argc) usage(argv[0]);
			s6d2_startup = argv[i];
		} else if (!strcmp(argv[i],"-s7d1")) {
			if (++i >= argc) usage(argv[0]);
			s7d1_startup = argv[i];
		} else if (!strcmp(argv[i],"-s7d2")) {
			if (++i >= argc) usage(argv[0]);
			s7d2_startup = argv[i];
		} else usage(argv[0]);
	}
	if ((err = EMUL_init())) exit(err);
	printf("\nResetting the emulator...\n");
	EMUL_reset();
	printf("\nRunning!\n");
	M65816_uperiod(4000);
	M65816_run();
}
